@@ -35,6 +35,28 @@
 
 G_BEGIN_DECLS
 
+#ifdef G_OS_WIN32
+
+/* Note that Win32 file permissions (in NTFS) are based on an ACL and
+ * not at all on anything like POSIX rwxrwxrwx bits. Additionally, a
+ * file can have a READONLY attribute. The stat() in the Microsoft C
+ * library only looks at the (MS-DOSish) READONLY attribute, not at
+ * all at the ACL. It fakes a st_mode based on the READONLY attribute
+ * only. If the FIXME below for GnomeVFSFilePermissions is fixed,
+ * these defines will become unnecessary.
+ */
+
+#define S_ISUID 0
+#define S_ISGID 0
+#define S_IRGRP (S_IRUSR >> 3)
+#define S_IWGRP (S_IWUSR >> 3)
+#define S_IXGRP (S_IXUSR >> 3)
+#define S_IROTH (S_IRUSR >> 6)
+#define S_IWOTH (S_IWUSR >> 6)
+#define S_IXOTH (S_IXUSR >> 6)
+
+#endif
+
 /**
  * GnomeVFSInodeNumber:
  *
@@ -103,6 +125,7 @@ typedef enum {
  * @GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE: Mime type field is valid
  * @GNOME_VFS_FILE_INFO_FIELDS_ACCESS: Access bits of the permissions
  * bitfield are valid
+ * @GNOME_VFS_FILE_INFO_FIELDS_IDS: UID and GID information are valid
  *
  * Flags indicating what fields in a GnomeVFSFileInfo struct are valid. 
  * Name is always assumed valid (how else would you have gotten a
@@ -125,7 +148,8 @@ typedef enum {
 	GNOME_VFS_FILE_INFO_FIELDS_CTIME = 1 << 11,
 	GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME = 1 << 12,
 	GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE = 1 << 13,
-	GNOME_VFS_FILE_INFO_FIELDS_ACCESS = 1 << 14
+	GNOME_VFS_FILE_INFO_FIELDS_ACCESS = 1 << 14,
+	GNOME_VFS_FILE_INFO_FIELDS_IDS = 1 << 15
 } GnomeVFSFileInfoFields;
 
 /* FIXME: It's silly to use the symbolic constants for POSIX here.
@@ -299,11 +323,18 @@ typedef enum {
 	GNOME_VFS_SET_FILE_INFO_TIME = 1 << 3
 } GnomeVFSSetFileInfoMask;
 
-typedef struct {
+
+struct _GnomeVFSGetFileInfoResult {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *file_info;
-} GnomeVFSGetFileInfoResult;
+};
+
+typedef struct _GnomeVFSGetFileInfoResult GnomeVFSGetFileInfoResult;
+
+GType                      gnome_vfs_get_file_info_result_get_type (void);
+GnomeVFSGetFileInfoResult* gnome_vfs_get_file_info_result_dup  (GnomeVFSGetFileInfoResult *result);
+void                       gnome_vfs_get_file_info_result_free (GnomeVFSGetFileInfoResult *result);
 
 /**
  * GNOME_VFS_FILE_INFO_SYMLINK:
