@@ -906,12 +906,17 @@ neon_session_save_auth (ne_request 	  *req,
 
 
 /* ************************************************************************** */
-/* Response header parser */
+/* Header parsing utilities */
+
 static gboolean
 header_value_to_number (const char *header_value, gulong *number)
 {
 	gulong result;
 	const char *p = header_value;
+
+	if (header_value == NULL) {
+		return FALSE;
+	}
 	
 	for (result = 0; g_ascii_isdigit (*p); p++)
 		result = 10 * result + (*p - '0');
@@ -923,21 +928,6 @@ header_value_to_number (const char *header_value, gulong *number)
 	return TRUE;
 }
 
-static void
-set_content_length (GnomeVFSFileInfo *file_info, const char *value)
-{
-	gboolean result;
-	gulong size;
-	
-	result = header_value_to_number (value, &size);
-	
-	if (! result) return;
-
-	DEBUG_HTTP_2 ("set size");
-	
-	file_info->size = size;
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
-}
 
 static char *
 strip_semicolon (const char *value)
@@ -952,44 +942,12 @@ strip_semicolon (const char *value)
 	}
 }
 
-static void
-set_content_type (GnomeVFSFileInfo *file_info, const char *value)
-{
-	g_free (file_info->mime_type);
 
-	file_info->mime_type = strip_semicolon (value);
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
-}
-
-static void
-set_last_modified (GnomeVFSFileInfo *file_info, const char *value)
-{
-	time_t time;
-
-	if (! gnome_vfs_atotm (value, &time))
-		return;
-	
-	file_info->mtime = time;
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MTIME;
-}
-
-static void
-set_access_time (GnomeVFSFileInfo *file_info, const char *value)
-{
-	time_t time;
-
-	if (! gnome_vfs_atotm (value, &time))
-		return;
-
-	file_info->atime = time;
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_ATIME;
-}
-
-static void 
-set_dav_class (void *userdata, const char *value)
+static DavClass 
+parse_dav_header (const char *value)
 {
 	char *tokens = ne_strdup(value), *pnt = tokens;
-        DavClass *dav_class = userdata;
+        DavClass dav_class = DAV_CLASS_NOT_SET;
     
 	DEBUG_HTTP_3 ("parsing dav: %s", tokens); 
 
@@ -1000,16 +958,17 @@ set_dav_class (void *userdata, const char *value)
 		tok = ne_shave(tok, " \r\t");
 
 		if (strcmp (tok, "1") == 0) {
-			*dav_class = 1;
+			dav_class = 1;
 		} else if (strcmp(tok, "2") == 0) {
-			*dav_class = 1;
+			dav_class = 1;
 		} 
 		
-		DEBUG_HTTP_3 ("DAV Level: %d", *dav_class);
+		DEBUG_HTTP_3 ("DAV Level: %d", dav_class);
 		
 	} while (pnt != NULL);
     
 	ne_free (tokens);
+	return dav_class;
 }
 
 typedef enum {
@@ -1074,15 +1033,16 @@ quit_allow_lookup_destroy (void)
 }
 
 
-static void 
-set_allow (void *userdata, const char *value)
+static HttpMethods
+parse_allow_header (const char *value)
 {
-	char *tokens = ne_strdup(value), *pnt = tokens;
-	HttpMethods *methods = userdata;
+	char *tokens = ne_strdup (value), *pnt = tokens;
+	HttpMethods methods;
 	struct HttpMethod *method;
     
 	DEBUG_HTTP_3 ("parsing allow: %s", pnt);
-	
+	methods = ALLOW_NOT_SET;
+
 	do {
 		char *tok = ne_qtoken (&pnt, ',',  "\"'");
 		if (!tok) break;
@@ -1096,21 +1056,12 @@ set_allow (void *userdata, const char *value)
 		
 		DEBUG_HTTP_3 ("setting %s to yes", method->string);
 		
-		*methods |= method->type;
+		methods |= method->type;
 		
 	} while (pnt != NULL);
     
 	ne_free (tokens);
-}
-
-static void 
-set_content_range (void *ud, const char *value)
-{
-	int *range = ud;
-	
-    	if (strncmp (value, "bytes ", 6) != 0) {
-		*range = -1;
-    	}
+	return methods;
 }
 
 /* function for setting the etag value
@@ -1126,28 +1077,49 @@ set_etag (void **etag, const char *value)
 */
 
 static void
-add_default_header_handlers (ne_request *req, GnomeVFSFileInfo *info)
+std_headers_to_file_info (ne_request *req, GnomeVFSFileInfo *info)
 {
+	const char *value;
+	time_t	    time;
+	gboolean    result;
+	gulong      size;
+	
+	value = ne_get_response_header (req, "Last-Modified");
+
+	if (value != NULL && gnome_vfs_atotm (value, &time)) {
+		
+		info->mtime = time;
+		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MTIME;
+	}
 
-	ne_add_response_header_handler (req, "Last-Modified", 
-					(ne_header_handler) set_last_modified,
-		   			info);
 	
-	ne_add_response_header_handler (req, "Content-Length", 
-					(ne_header_handler) set_content_length,
-					info);
+	value  = ne_get_response_header (req, "Content-Length");
+	result = header_value_to_number (value, &size);
 	
-	ne_add_response_header_handler (req, "Content-Type",
-					(ne_header_handler) set_content_type,
-					info);
+	if (result == TRUE) {
 	
-	ne_add_response_header_handler (req, "Date",
-					(ne_header_handler) set_access_time,
-					info);
-       
+		info->size = size;
+		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
+	}
+	
+	
+	value  = ne_get_response_header (req, "Content-Type");
+
+	if (value != NULL) {
+		g_free (info->mime_type);
+
+		info->mime_type = strip_semicolon (value);
+		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+	}
+
+	value = ne_get_response_header (req, "Date");
+
+	if (value != NULL && gnome_vfs_atotm (value, &time)) {
+		info->atime = time;
+		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_ATIME;
+	}
 }
 
-
 /* ************************************************************************** */
 /* Propfind request handlers */
 static const ne_propname file_info_props[] = {
@@ -1422,6 +1394,11 @@ neon_session_pool_insert (GnomeVFSURI *uri, ne_session *session)
 		g_hash_table_insert (neon_session_table, uri, pool);
 	}
 	
+	/* Make sure we forget the auth info so multiple users for
+	 * the same uri doesnt confict. We have our own cache to
+	 * compensate this */
+	ne_forget_auth (session);	
+	
 	pool->unused_sessions = g_list_append (pool->unused_sessions, 
 					       session);
 
@@ -1474,38 +1451,55 @@ neon_return_headers (ne_request *req, void *userdata, const ne_status *status)
 {	
 	GnomeVFSModuleCallbackReceivedHeadersIn in_args;
 	GnomeVFSModuleCallbackReceivedHeadersOut out_args;
-	GList **headers, *iter;
+	GList       *headers;
+	void        *hiter;
 	GnomeVFSURI *uri;
-	ne_session *session;
+	ne_session  *session;
+	const char  *hkey, *hval;
 
 	DEBUG_HTTP_FUNC (1);
 	
 	session = ne_get_session (req);
 	
-	if (ne_get_request_private (req, "Headers Returned"))
+	if (ne_get_request_private (req, "Headers Returned")) {
 		return 0;
+	}
+	
+	headers = NULL;
+	hiter = NULL;
+
+	while ((hiter = ne_response_header_iterate (req,
+						    hiter,
+						    &hkey,
+						    &hval))) {
+		char *header;
+
+		if (hkey == NULL || hval == NULL) {
+			continue;
+		}
+		
+		header = g_strdup_printf ("%s: %s", hkey, hval);
+		headers = g_list_prepend (headers, header);
+	}
+
+	if (headers == NULL) {
+		return 0;
+	}
 	
-	headers = ne_get_request_private (req, "Headers");
 	uri = ne_get_session_private (session, "GnomeVFSURI");
 
 	memset (&in_args, 0, sizeof (in_args));
 	memset (&out_args, 0, sizeof (out_args));
 	
 	in_args.uri = uri;
-	in_args.headers = *headers;
+	in_args.headers = headers;
 
 	gnome_vfs_module_callback_invoke (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS,
 					&in_args, sizeof (in_args),
 					&out_args, sizeof (out_args));
 	
-	for (iter = *headers; iter; iter = iter->next) {
-		DEBUG_HTTP_3 ("Headers returned %s,", (char *) iter->data);
-		g_free (iter->data);
-	}
-
-	g_list_free (*headers);
-	g_free (headers);
-
+	/* FIXME: free stuff here?  */
+	
 	ne_set_request_private (req, "Headers Returned", "TRUE");
 	
 	DEBUG_HTTP_FUNC (0);
@@ -1513,34 +1507,18 @@ neon_return_headers (ne_request *req, void *userdata, const ne_status *status)
 	return 0;
 }
 
-static void 
-neon_header_catcher (void *userdata, const char *value)
-{
-	GList **headers = (GList **) userdata;
-
-	DEBUG_HTTP_3 ("Catching Header %s,", (char *) value);
-
-	*headers = g_list_prepend (*headers, g_strdup (value));	
-}
-
-
 static void 
 neon_setup_headers (ne_request *req, void *userdata, ne_buffer *header)
 {
 	GnomeVFSModuleCallbackAdditionalHeadersIn in_args;
 	GnomeVFSModuleCallbackAdditionalHeadersOut out_args;
-	GList *iter, **headers;
+	GList *iter;
 	gboolean ret;
 	GnomeVFSURI *uri;
 	ne_session *session;
 
 	DEBUG_HTTP_FUNC (1);
 	
-	headers = g_new0 (GList *, 1);
-
-	ne_set_request_private (req, "Headers", headers);
-	ne_add_response_header_catcher (req, neon_header_catcher, headers);
-
 	session = ne_get_session (req);
 	uri = ne_get_session_private (session, "GnomeVFSURI");
 
@@ -1573,25 +1551,25 @@ neon_setup_headers (ne_request *req, void *userdata, ne_buffer *header)
 typedef struct {
 	
 	GnomeVFSURI *uri;
-	char *path;
+	char        *path;
 	
 	const char *scheme;
-	gboolean ssl;
+	gboolean    ssl;
 	
-	DavClass dav_class;
+	DavClass    dav_class;
 	HttpMethods methods;
 	
 	ne_session *session;
 
-	gboolean dav_mode;
-	gboolean redirected;
-	guint    redir_count;
+	gboolean    dav_mode;
+	gboolean    redirected;
+	guint       redir_count;
 	
 } HttpContext;
 
 
 static GnomeVFSResult
-http_aquire_connection (HttpContext *context)
+http_acquire_connection (HttpContext *context)
 {
 	GnomeVFSToplevelURI *top_uri;
 	char *user_agent;	
@@ -1750,7 +1728,7 @@ http_context_open (GnomeVFSURI *uri, HttpContext **context)
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 	
-	result = http_aquire_connection (ctx);
+	result = http_acquire_connection (ctx);
 	
 	if (result != GNOME_VFS_OK) {
 		*context = NULL;
@@ -1758,7 +1736,10 @@ http_context_open (GnomeVFSURI *uri, HttpContext **context)
 		return result;
 	}
 
-	ctx->dav_mode = scheme_is_dav (uri);
+	/* initialize the missing bits */
+	ctx->dav_mode  = scheme_is_dav (uri);
+	ctx->dav_class = DAV_CLASS_NOT_SET;
+	ctx->methods   = ALLOW_NOT_SET;
 	
 	*context = ctx;
 
@@ -1786,6 +1767,8 @@ http_follow_redirect (HttpContext *context)
 	
 	NE_FREE (redir_texturi);
 	
+	/* see if redirect is to another host:port pair so we need a new
+	 * connection/session */
 	if (! http_session_uri_equal (context->uri, new_uri)) {
 		
 		/* release connection */
@@ -1793,8 +1776,7 @@ http_follow_redirect (HttpContext *context)
 		context->session = NULL;
 		
 		http_context_set_uri (context, new_uri);
-		
-		result = http_aquire_connection (context);
+		result = http_acquire_connection (context);
 		
 	} else {
 		
@@ -1802,6 +1784,10 @@ http_follow_redirect (HttpContext *context)
 		ne_set_session_private (context->session, "GnomeVFSURI", context->uri);
 		result = GNOME_VFS_OK;
 	}
+	
+	/* uri got dupped by http_context_set_uri () */
+	gnome_vfs_uri_unref (new_uri);
+	
 	DEBUG_HTTP ("[Redirec] Redirect result: %s", 
 		    gnome_vfs_result_to_string (result));
 	return result;
@@ -1854,7 +1840,6 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 	   will treat PROPFIND as GET and some servers may deny us PROFIND but
 	   allow us HEAD. */
 	if (res == NE_OK && ne_get_status (req)->code == 207) {
-		
 
 		if (pfctx.target != NULL) {
 			gnome_vfs_file_info_copy (info, pfctx.target);	
@@ -1872,8 +1857,6 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
  head_start:	
 	req  = ne_request_create (context->session, "HEAD", context->path);
 
-	add_default_header_handlers (req, info);
-		
 	res = ne_request_dispatch (req);
 
 	if (res == NE_REDIRECT) {
@@ -1891,9 +1874,11 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 	ne_request_destroy (req);
 	
 	if (result == GNOME_VFS_OK) {
-		const char *name;
+		const char *name;	
 		
 		name = gnome_vfs_uri_get_path (context->uri);
+	
+		gnome_vfs_file_info_clear (info);
 		
 		info->name  = g_path_get_basename (name);
 		info->type  = GNOME_VFS_FILE_TYPE_REGULAR;
@@ -1901,8 +1886,11 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 		
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
 
+		std_headers_to_file_info (req, info);
+		
 		/* work-around for broken icecast server */
-		if (! g_ascii_strcasecmp (info->mime_type, "audio/mpeg")) {
+		if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE
+		    && ! g_ascii_strcasecmp (info->mime_type, "audio/mpeg")) {
 			ne_close_connection (ne_get_session (req));
 		}
 		
@@ -1972,17 +1960,10 @@ http_options (HttpContext *hctx)
 {
 	GnomeVFSResult  result;
 	ne_request     *req;
-	DavClass 	klass;
-	HttpMethods	methods;
 	int		res;
 	
  options_start:	
-	klass = DAV_CLASS_NOT_SET;
-	methods = 0;
 	req = ne_request_create (hctx->session, "OPTIONS", hctx->path);
-	ne_add_response_header_handler (req, "DAV", set_dav_class, &klass);
-	ne_add_response_header_handler (req, "Allow", set_allow, &methods);
-	
 	res = ne_request_dispatch (req);
 	
 	if (res == NE_REDIRECT) {
@@ -1992,14 +1973,25 @@ http_options (HttpContext *hctx)
 	}
 
 	result = resolve_result (res, req);
-	ne_request_destroy (req);
-	
-	if (result == GNOME_VFS_OK) {
 		
-		hctx->dav_class = klass;
-		hctx->methods = methods;
+	if (result == GNOME_VFS_OK) {
+		const char *value;
+
+		value = ne_get_response_header (req, "DAV");
+
+		if (value != NULL) {
+			hctx->dav_class = parse_dav_header (value);
+		}
+	
+		value = ne_get_response_header (req, "Allow");
+
+		if (value != NULL) {
+			hctx->methods = parse_allow_header (value);
+		}
 	}
 
+	ne_request_destroy (req);
+
 	return result;
 }
 
@@ -2039,7 +2031,7 @@ dav_request (ne_request *req, gboolean allow_redirect)
 	status = (ne_status *) ne_get_status (req);
     
 	if (status->code == 207) {
-		if (!ne_xml_valid (p))
+		if (ne_xml_failed (p))
 			res = NE_ERROR;  
 		if (error != 0) {
 			status->code = error;
@@ -2200,8 +2192,6 @@ http_transfer_start_read (HttpFileHandle *handle)
 	GnomeVFSResult result;
 	HttpContext *hctx;
 	ne_request  *req;
-	int	     range;
-	char 	     dropbuf[4096];
 	int 	     res;
 	const ne_status *status;
 	
@@ -2217,19 +2207,17 @@ get_start:
 		
 		handle->can_range = TRUE;
 		
-		ne_print_request_header (req, "Range", "bytes=%"GNOME_VFS_OFFSET_FORMAT_STR"-", handle->offset);
-		
-		ne_add_response_header_handler (req, "Content-Range",
-						(ne_header_handler) set_content_range,
-						&range);
-	}
-	
+		ne_print_request_header (req, "Range",
+					 "bytes=%"GNOME_VFS_OFFSET_FORMAT_STR"-",
+					 handle->offset);
 
-	add_default_header_handlers (req, handle->info);
-	
-	res = ne_begin_request (req);
+		ne_add_request_header (req, "Accept-Ranges", "bytes");
+	}
+
+get_retry:
+
+	res    = ne_begin_request (req);
 	result = resolve_result (res, req);
-	
 	status = ne_get_status (req);
 	
 	DEBUG_HTTP ("[GET] %s, %d, %d", gnome_vfs_result_to_string (result), res, status->code);
@@ -2242,9 +2230,8 @@ get_start:
 	}
 	
 	if (IS_REDIRECT (status->code) || IS_AUTH_REQ (status->code)) {
-		/* send the body to /dev/null */				
-		while ((res = ne_read_response_block (req, dropbuf, sizeof (dropbuf))) > 0)
-			/* noop */;
+		/* We are not interested in the body */
+		res = ne_discard_response (req);	
 		
 		if (res < 0) {
 			handle->transfer_state = TRANSFER_ERROR;
@@ -2255,12 +2242,14 @@ get_start:
 		}
 		
 		res = ne_end_request (req);
+
+		if (res == NE_RETRY) {
+			goto get_retry;
+		}
+		
 		ne_request_destroy (req);
 		req = NULL;
 		
-		if (res == NE_RETRY || res == NE_AUTH || res == NE_PROXYAUTH)
-			goto get_start;
-		
 		if (res == NE_REDIRECT) {
 			result = http_follow_redirect (hctx);
 				
@@ -2271,6 +2260,8 @@ get_start:
 	
 	if (result == GNOME_VFS_OK) { 
 		/* 2xx .. success */
+	
+		std_headers_to_file_info (req, handle->info);
 		
 		if (handle->use_range && status->code != 206) {
 			DEBUG_HTTP ("[GET] {ranged} disabled");
@@ -2283,7 +2274,6 @@ get_start:
 		*/
 		neon_return_headers (req, NULL, status);
 
-		
 		handle->transfer_state = TRANSFER_READ;
 		handle->transfer.read = req;
 	}
@@ -2489,7 +2479,7 @@ do_close (GnomeVFSMethod 	*method,
 		   
 		DEBUG_HTTP ("[PUT] Filesize: %d", len);
 		
-		ne_set_request_body_buffer (req, data, len);
+		ne_set_request_body_buffer (req, (char *)data, len);
 		res = ne_request_dispatch (req);
 
 		result = resolve_result (res, req);
@@ -2703,7 +2693,7 @@ do_seek (GnomeVFSMethod	      *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod       *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset   *offset_return)
+	 GnomeVFSFileSize   *offset_return)
 {
 	HttpFileHandle *handle;
 	
@@ -3169,10 +3159,6 @@ do_set_file_info (GnomeVFSMethod 		*method,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 	
-	/* this wouldn't rename but move the file */
-	if (g_strrstr (info->name, "/"))
-		return GNOME_VFS_ERROR_BAD_PARAMETERS;
-
 	parent_uri = gnome_vfs_uri_get_parent (uri);
 	
 	if (parent_uri == NULL) {
